package com.nexblocks.authguard.jwt;

import com.google.inject.Inject;
import com.nexblocks.authguard.service.auth.AuthProvider;
import com.nexblocks.authguard.service.auth.ProvidesToken;
import com.nexblocks.authguard.service.model.*;

@ProvidesToken("oidc")
public class OpenIdConnectTokenProvider implements AuthProvider {
    private final AccessTokenProvider accessTokenProvider;
    private final IdTokenProvider idTokenProvider;

    @Inject
    public OpenIdConnectTokenProvider(final AccessTokenProvider accessTokenProvider,
                                      final IdTokenProvider idTokenProvider) {
        this.accessTokenProvider = accessTokenProvider;
        this.idTokenProvider = idTokenProvider;
    }

    @Override
    public AuthResponseBO generateToken(final AccountBO account) {
        return generateToken(account, null, null);
    }

    @Override
    public AuthResponseBO generateToken(final AccountBO account, final TokenOptionsBO options) {
        return generateToken(account, null, options);
    }

    @Override
    public AuthResponseBO generateToken(final AccountBO account, final TokenRestrictionsBO restrictions) {
        return generateToken(account, restrictions, null);
    }

    @Override
    public AuthResponseBO generateToken(final AccountBO account, final TokenRestrictionsBO restrictions,
                                        final TokenOptionsBO options) {
        final AuthResponseBO accessToken = accessTokenProvider.generateToken(account, restrictions, options);
        final AuthResponseBO idToken = idTokenProvider.generateToken(account);

        return AuthResponseBO.builder()
                .type("oidc")
                .entityId(account.getId())
                .entityType(EntityType.ACCOUNT)
                .token(OAuthResponseBO.builder()
                        .accessToken((String) accessToken.getToken())
                        .idToken((String) idToken.getToken())
                        .refreshToken((String) accessToken.getRefreshToken())
                        .build())
                .build();
    }

    @Override
    public AuthResponseBO generateToken(final AppBO app) {
        throw new UnsupportedOperationException("OpenID Connect tokens cannot be generated for applications");
    }
}
